"""Module for XKNX Exception handling."""
# flake8: noqa
from .exception import XKNXException, ConversionError, \
    CouldNotParseTelegram, CouldNotParseKNXIP, CouldNotParseAddress, \
    DeviceIllegalValue
